# -*- coding: utf-8 -*-

#######################################################################
# DO NOT CHANGE SETTINGS IN THIS FILE, ONLY ADD NEW SENSIBLE DEFAULTS #
# FOR MORE INFORMATION SEE local_settings_template.py.                #
#######################################################################

import os

gettext = lambda s: s

########################
# MAIN DJANGO SETTINGS #
########################

ADMINS = (
        ('Miguel Vieira', 'jmvieira@gmail.com'),
        )

MANAGERS = ADMINS

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
USE_TZ = True
TIME_ZONE = 'Europe/London'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-gb'

DEBUG = False
TEMPLATE_DEBUG = DEBUG

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'vpa6d3#8v6#l27_p*4luyl%-4ldezbr#8&amp;6slkxbokign_w5@&amp;'

# Tuple of IP addresses, as strings, that:
#   * See debug comments, when DEBUG is true
#   * Receive x-headers
INTERNAL_IPS = ('127.0.0.1',)

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
        #     'django.template.loaders.eggs.Loader',
        )

#############
# DATABASES #
#############

DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.',
            'NAME': '',
            'USER': '',
            'PASSWORD': '',
            'HOST': '',
            'PORT': '',
            }
        }

#########
# PATHS #
#########

PROJECT_URL = '/'
LOGIN_URL = PROJECT_URL + 'accounts/login/'
LOGIN_URL_REDIRECT = PROJECT_URL + 'eats/search/'

# Full filesystem path to the project.
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))

# Name of the directory for the project.
PROJECT_DIRNAME = PROJECT_ROOT.split(os.sep)[-1]

# Every cache key will get prefixed with this value - here we set it to
# the name of the directory the project is in to try and use something
# project specific.
CACHE_MIDDLEWARE_KEY_PREFIX = PROJECT_DIRNAME

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = '/media/'

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = os.path.join(PROJECT_ROOT, MEDIA_URL.strip('/'))

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

ADMIN_MEDIA_PREFIX = STATIC_URL + 'grappelli/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(PROJECT_ROOT, STATIC_URL.strip('/'))

# Additional locations of static files
STATICFILES_DIRS = (
        # Put strings here, like "/home/html/static" or "C:/www/django/static".
        # Always use forward slashes, even on Windows.
        # Don't forget to use absolute paths, not relative paths.
        )

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
        'django.contrib.staticfiles.finders.FileSystemFinder',
        'django.contrib.staticfiles.finders.AppDirectoriesFinder',
        #    'django.contrib.staticfiles.finders.DefaultStorageFinder',
        )


AUTHENTICATION_BACKENDS = [
        'django.contrib.auth.backends.ModelBackend',
        # Authentication using REMOTE_USER
        'django.contrib.auth.backends.RemoteUserBackend'
        ]

ROOT_URLCONF = 'ereed.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'ereed.wsgi.application'

TEMPLATE_DIRS = (
        # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
        # Always use forward slashes, even on Windows.
        # Don't forget to use absolute paths, not relative paths.
        os.path.join(PROJECT_ROOT, 'templates'),
        )

################
# APPLICATIONS #
################

INSTALLED_APPS = (
        'django.contrib.admin',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.redirects',
        'django.contrib.sessions',
        'django.contrib.sites',
        'django.contrib.sitemaps',
        'django.contrib.staticfiles',
        'mezzanine.boot',
        'mezzanine.conf',
        'mezzanine.core',
        'mezzanine.generic',
        'mezzanine.blog',
        'mezzanine.forms',
        'mezzanine.pages',
        'mezzanine.galleries',
        'mezzanine.twitter',
        'mezzanine.accounts',
        #'mezzanine.mobile',
        )

TEMPLATE_CONTEXT_PROCESSORS = (
        'django.contrib.auth.context_processors.auth',
        'django.contrib.messages.context_processors.messages',
        'django.core.context_processors.debug',
        'django.core.context_processors.i18n',
        'django.core.context_processors.static',
        'django.core.context_processors.media',
        'django.core.context_processors.request',
        'django.core.context_processors.tz',
        'mezzanine.conf.context_processors.settings',
        )

MIDDLEWARE_CLASSES = (
        'mezzanine.core.middleware.UpdateCacheMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        # Authentication using REMOTE_USER
        'django.contrib.auth.middleware.RemoteUserMiddleware',
        'django.contrib.redirects.middleware.RedirectFallbackMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'mezzanine.core.request.CurrentRequestMiddleware',
        'mezzanine.core.middleware.TemplateForDeviceMiddleware',
        'mezzanine.core.middleware.TemplateForHostMiddleware',
        'mezzanine.core.middleware.AdminLoginInterfaceSelectorMiddleware',
        # Uncomment the following if using any of the SSL settings:
        # 'mezzanine.core.middleware.SSLRedirectMiddleware',
        'mezzanine.pages.middleware.PageMiddleware',
        'mezzanine.core.middleware.FetchFromCacheMiddleware',
        )

# Store these package names here as they may change in the future since
# at the moment we are using custom forks of them.
PACKAGE_NAME_FILEBROWSER = 'filebrowser_safe'
PACKAGE_NAME_GRAPPELLI = 'grappelli_safe'

#########################
# APPLICATIONS SETTINGS #
#########################

# Commonplacer Applications
INSTALLED_APPS = INSTALLED_APPS + (
        'selectable',
        'tmapi',
        'eats',
        PACKAGE_NAME_FILEBROWSER,
        PACKAGE_NAME_GRAPPELLI,
        )

# EATS
EATS_TOPIC_MAP = 'http://ereed.cch.kcl.ac.uk/'

# FileBrowser
FILEBROWSER_EXTENSIONS = {
    'Folder': [''],
    'Image': ['.jpg','.jpeg','.gif','.png','.tif','.tiff'],
    'Document': ['.pdf','.doc','.rtf','.txt','.xls','.csv','.zip'],
    'Video': ['.mov','.wmv','.mpeg','.mpg','.avi','.rm'],
    'Audio': ['.mp3','.mp4','.wav','.aiff','.midi','.m4p']
}

# Grappelli
GRAPPELLI_ADMIN_TITLE = 'eREED'

# Mezzanine
ACCOUNTS_VERIFICATION_REQUIRED = True
SITE_TITLE = 'The Fortune Theatre Records'
SITE_TAGLINE = 'Welcome to a Prototype Digital Edition by Records of Early English Drama (REED).'

# South
USE_SOUTH = True
SOUTH_TESTS_MIGRATE = False

#########################
# OPTIONAL APPLICATIONS #
#########################

# These will be added to ``INSTALLED_APPS``, only if available.
OPTIONAL_APPS = (
        'debug_toolbar',
        'django_extensions'
        )

DEBUG_TOOLBAR_CONFIG = {'INTERCEPT_REDIRECTS': False}

for app in OPTIONAL_APPS:
    try:
        __import__(app)
    except ImportError:
        pass
    else:
        INSTALLED_APPS += (app,)

if 'debug_toolbar' in INSTALLED_APPS:
    MIDDLEWARE_CLASSES += (
            'debug_toolbar.middleware.DebugToolbarMiddleware',
            )

    ########
# LOGS #
########

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
        'version': 1,
        'disable_existing_loggers': False,
        'handlers': {
            'mail_admins': {
                'level': 'ERROR',
                'class': 'django.utils.log.AdminEmailHandler'
                }
            },
        'loggers': {
            'django.request': {
                'handlers': ['mail_admins'],
                'level': 'ERROR',
                'propagate': True,
                },
            }
        }

##################
# LOCAL SETTINGS #
##################

# Allow any settings to be defined in local_settings.py which should be ignored
# from version control system allowing for settings to be defined per machine.
try:
    from local_settings import *
except ImportError:
    pass

####################
# DYNAMIC SETTINGS #
####################
# set_dynamic_settings() will rewrite globals based on what has been
# defined so far, in order to provide some better defaults where
# applicable. We also allow this settings module to be imported
# without Mezzanine installed, as the case may be when using the
# fabfile, where setting the dynamic settings below isn't strictly
# required.
try:
    from mezzanine.utils.conf import set_dynamic_settings
except ImportError:
    pass
else:
    set_dynamic_settings(globals())
