<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet exclude-result-prefixes="#all" version="2.0"
  xmlns:eats="http://eats.artefact.org.nz/ns/eatsml/" xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p><xd:b>Created on:</xd:b> Nov 4, 2010</xd:p>
      <xd:p><xd:b>Author:</xd:b> jvieira</xd:p>
      <xd:p>Normalizes the EATSML for Solr indexing.</xd:p>
    </xd:desc>
  </xd:doc>

  <xsl:template match="/">
    <entities>
      <xsl:apply-templates select="eats:collection/eats:entities" />
    </entities>
  </xsl:template>

  <xsl:template match="eats:entity">
    <xsl:variable name="entity-id" select="@xml:id" />

    <entity url="{@url}" xml:id="{$entity-id}">
      <types>
        <xsl:apply-templates select="eats:entity_types" mode="entity" />
      </types>
      <names>
        <xsl:apply-templates select="eats:names" />
      </names>
      <relationships>
        <xsl:apply-templates select="eats:entity_relationships" />
      </relationships>
      <identifiers>
        <xsl:apply-templates select="eats:subject_identifiers" />
      </identifiers>
    </entity>
  </xsl:template>

  <xsl:template match="eats:entity_type" mode="entity">
    <type>
      <xsl:apply-templates select="id(@entity_type)" />
    </type>
  </xsl:template>
  
  <xsl:template match="eats:entity_type">
      <xsl:value-of select="normalize-unicode(eats:name)" />
  </xsl:template>

  <xsl:template match="eats:name">
    <name>
      <xsl:attribute name="type">
        <xsl:apply-templates select="id(@name_type)" />
      </xsl:attribute>
      <xsl:if test="@is_preferred = 'true'">
        <xsl:attribute name="is_preferred">
          <xsl:value-of select="true()" />
        </xsl:attribute>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="normalize-space(eats:display_form)">
          <xsl:value-of select="normalize-space(eats:display_form)" />
        </xsl:when>
        <xsl:when test="normalize-space(eats:assembled_form)">
          <xsl:value-of select="normalize-space(eats:assembled_form)" />
        </xsl:when>
      </xsl:choose>
    </name>
  </xsl:template>

  <xsl:template match="eats:name_type">
    <xsl:value-of select="normalize-space(eats:name)" />
  </xsl:template>

  <xsl:template match="eats:entity_relationship">
    <relationship>
      <xsl:attribute name="direction">
        <xsl:choose>
          <xsl:when test="@domain_entity = ancestor::eats:entity/@xml:id">direct</xsl:when>
          <xsl:otherwise>indirect</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:apply-templates select="id(@entity_relationship_type)" />
      </xsl:attribute>

      <xsl:choose>
        <xsl:when test="@domain_entity = ancestor::eats:entity/@xml:id">
          <xsl:apply-templates mode="related-entity" select="id(@range_entity)" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates mode="related-entity" select="id(@domain_entity)" />
        </xsl:otherwise>
      </xsl:choose>
    </relationship>
  </xsl:template>
  
  <xsl:template match="eats:entity_relationship_type">
    <xsl:value-of select="normalize-space(eats:name)" />
  </xsl:template>

  <xsl:template match="eats:entity" mode="related-entity">
    <entity url="{@url}" xml:id="{@xml:id}">
      <xsl:apply-templates mode="related-entity" select="eats:names" />
    </entity>
  </xsl:template>

  <xsl:template match="eats:names" mode="related-entity">
    <name>
      <xsl:choose>
        <xsl:when test="eats:name/@is_preferred = 'true'">
          <xsl:value-of
            select="normalize-space(eats:name[@is_preferred = 'true']/eats:display_form)" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="normalize-space(eats:name[1]/eats:display_form)" />
        </xsl:otherwise>
      </xsl:choose>
    </name>
  </xsl:template>

  <xsl:template match="eats:entity_relationship" mode="inverse">
    <relationship direction="inverse">
      <xsl:attribute name="type">
        <xsl:apply-templates select="id(@entity_relationship_type)" />
      </xsl:attribute>

      <xsl:apply-templates mode="related-entity" select="ancestor::eats:entity[1]" />
    </relationship>
  </xsl:template>
  
  <xsl:template match="eats:subject_identifier">
    <identifier>
      <xsl:value-of select="normalize-space(.)" />
    </identifier>
  </xsl:template>
</xsl:stylesheet>
