<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet exclude-result-prefixes="#all" version="2.0"
  xmlns:eats="http://eats.artefact.org.nz/ns/eatsml/" xmlns:tei="http://www.tei-c.org/ns/1.0"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:import href="../../kiln/stylesheets/solr/tei-eats-to-solr.xsl" />

  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p><xd:b>Created on:</xd:b> Nov 3, 2010</xd:p>
      <xd:p><xd:b>Author:</xd:b> jvieira</xd:p>
      <xd:p>This stylesheet converts a TEI document together with an EATSML document into a Solr
        index document. It expects the parameter file-path, which is the path of the file being
        indexed.</xd:p>
    </xd:desc>
  </xd:doc>

  <!-- Path to the TEI file being indexed. -->
  <xsl:param name="file-path" />
  <xsl:key match="entities/entity" name="entity-by-key" use="@url" />

  <xsl:template match="/">
    <add>
      <xsl:apply-templates />
    </add>
  </xsl:template>

  <xsl:template match="tei:TEI">
    <xsl:variable name="document-metadata">
      <xsl:apply-templates mode="document-metadata" select="tei:teiHeader" />
    </xsl:variable>

    <xsl:apply-templates select="tei:text/tei:group/tei:text[@xml:id]">
      <xsl:with-param name="document-metadata" select="$document-metadata" tunnel="yes" />
      <xsl:with-param name="file-path" select="$file-path" tunnel="yes" />
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="tei:fileDesc/tei:titleStmt" mode="document-metadata">
    <field name="document_title">
        <xsl:for-each select="tei:title">
            <xsl:value-of select="." />
            <xsl:if test="position != last()">
                <xsl:text>: </xsl:text>
            </xsl:if>
        </xsl:for-each>
    </field>
    <xsl:apply-templates select="tei:author" />
    <xsl:apply-templates select="tei:editor" />
  </xsl:template>

  <xsl:template match="tei:text[@xml:id]">
    <xsl:param name="document-metadata" tunnel="yes" />

    <xsl:variable name="document-id" select="ancestor::tei:TEI/@xml:id" />
    <doc>
      <xsl:sequence select="$document-metadata" />
      <field name="document_id">
        <xsl:value-of select="$document-id" />
      </field>
      <field name="file">
        <xsl:value-of select="$file-path" />
      </field>
      <field name="record_id">
        <xsl:value-of select="@xml:id" />
      </field>
      <field name="record_title">
        <xsl:value-of select="tei:body/tei:head[@type = 'main']" />
      </field>
      <field name="record_subtitle">
        <xsl:value-of select="tei:body/tei:head[@type = 'sub']" />
      </field>
      <field name="document_type">
        <xsl:value-of select="@type" />
      </field>

      <xsl:apply-templates select="tei:body/tei:head[@type = 'main']/tei:date" />
      <xsl:apply-templates mode="entity-mention-by-tei" select=".//tei:*[@key]" />

      <xsl:variable name="free-text">
        <xsl:apply-templates mode="free-text" />
      </xsl:variable>

      <field name="text">
        <xsl:value-of select="normalize-space($free-text)" />
      </field>
    </doc>
  </xsl:template>

  <xsl:template match="tei:date">
    <xsl:if test="@when-iso">
      <field name="year">
        <xsl:value-of select="substring(@when-iso, 1, 4)" />
      </field>
      <field name="year_sort">
        <xsl:value-of select="substring(@when-iso, 1, 4)" />
      </field>
    </xsl:if>
    <xsl:if test="@notBefore-iso">
      <field name="year">
        <xsl:value-of select="substring(@notBefore-iso, 1, 4)" />
      </field>
      <field name="year_sort">
        <xsl:value-of select="substring(@notBefore-iso, 1, 4)" />
      </field>
    </xsl:if>
    <xsl:if test="@notAfter-iso">
      <field name="year">
        <xsl:value-of select="substring(@notAfter-iso, 1, 4)" />
      </field>
    </xsl:if>
    <xsl:if test="@from-iso and @to-iso">
      <xsl:for-each
        select="xs:integer(substring(@from-iso, 1, 4)) to xs:integer(substring(@to-iso, 1, 4))">
        <field name="year">
          <xsl:value-of select="." />
        </field>
      </xsl:for-each>
      <field name="year_sort">
        <xsl:value-of select="substring(@from-iso, 1, 4)" />
      </field>
    </xsl:if>
  </xsl:template>

  <xsl:template match="tei:*[@key]" mode="entity-mention-by-tei">
    <xsl:if test="normalize-space(@type)">
      <field name="entity_url">
        <xsl:value-of select="@key" />
      </field>
      <field name="entity_type">
        <xsl:value-of select="@type" />
      </field>
      <field name="entity_name">
        <xsl:value-of select="normalize-space(.)" />
      </field>
      <field name="{@type}_name">
        <xsl:value-of select="normalize-space(.)" />
      </field>

      <xsl:apply-templates mode="entity-mention-by-tei" select="key('entity-by-key', @key)" />
    </xsl:if>
  </xsl:template>

  <xsl:template match="entities/entity" mode="entity-mention-by-tei">
    <xsl:variable name="entity-id" select="@xml:id" />
    <field name="entity_key">
      <xsl:value-of select="$entity-id" />
    </field>

    <xsl:apply-templates mode="entity-mention-by-tei" select="names/name" />
  </xsl:template>

  <xsl:template match="names/name" mode="entity-mention-by-tei">
    <xsl:variable name="name" select="." />

    <xsl:if test="@is_preferred = 'true'">
      <field name="entity">
        <xsl:value-of select="ancestor::entity/@xml:id" />
        <xsl:text>::</xsl:text>
        <xsl:value-of select="$name" />
      </field>
      <field name="entity_display_name">
        <xsl:value-of select="$name" />
      </field>

      <xsl:for-each select="../../types/type">
        <field name="{.}_entity">
          <xsl:value-of select="ancestor::entity/@xml:id" />
          <xsl:text>::</xsl:text>
          <xsl:value-of select="$name" />
        </field>
        <field name="{.}_display_name">
          <xsl:value-of select="$name" />
        </field>
      </xsl:for-each>
    </xsl:if>
    <field name="entity_name">
      <xsl:value-of select="$name" />
    </field>
  </xsl:template>

  <!-- EATS entities-->
  <xsl:template match="entity[not(names/name)]" />

  <xsl:template match="entity[names/name]">
    <xsl:variable name="entity-url" select="@url" />

    <doc>
      <field name="document_type">entity</field>
      <field name="record_id">
        <xsl:value-of select="@xml:id" />
      </field>

      <field name="entity_key">
        <xsl:value-of select="@xml:id" />
      </field>
      <field name="entity_url">
        <xsl:value-of select="$entity-url" />
      </field>

      <!-- Entity data, drawn from EATS. -->
      <xsl:apply-templates mode="entity-by-entity" />

      <!-- Entity references, drawn from the TEI. -->
      <xsl:apply-templates mode="entity-mention-by-entity"
        select="//tei:TEI/tei:text/tei:group/tei:text[@xml:id]//tei:*[@key = $entity-url]">
        <xsl:with-param name="entity-key" select="$entity-url" />
      </xsl:apply-templates>
    </doc>
  </xsl:template>

  <xsl:template match="names" mode="entity-by-entity">
    <field name="document_title">
      <xsl:call-template name="get-preferred-name">
        <xsl:with-param name="names" select="." />
      </xsl:call-template>
    </field>

    <xsl:apply-templates mode="entity-by-entity" select="name" />
  </xsl:template>


  <xsl:template name="get-preferred-name">
    <xsl:param name="names" required="yes" />
    <xsl:choose>
      <xsl:when test="$names/name[@is_preferred = true()]">
        <xsl:value-of select="$names/name[@is_preferred = true()][1]" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$names/name[1]" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="names/name" mode="entity-by-entity">
    <field name="entity_display_name">
      <xsl:value-of select="." />
    </field>
  </xsl:template>

  <xsl:template match="types/type" mode="entity-by-entity">
    <field name="entity_type">
      <xsl:value-of select="." />
    </field>
  </xsl:template>

  <xsl:template match="relationships/relationship" mode="entity-by-entity">
    <field name="{@direction}_relationship">
      <xsl:value-of select="@type" />
      <xsl:text>::</xsl:text>
      <xsl:value-of select="entity/@xml:id" />
      <xsl:text>::</xsl:text>
      <xsl:value-of select="entity/name" />
      <xsl:text>::</xsl:text>
      <xsl:value-of select="//entity[@xml:id = current()/entity/@xml:id][types]/types/type[1]" />
    </field>
  </xsl:template>

  <xsl:template match="identifiers/identifier" mode="entity-by-entity">
    <field name="identifier">
      <xsl:value-of select="." />
    </field>
  </xsl:template>

  <!-- Entity instances in the TEI. -->
  <xsl:template match="tei:*[@key]" mode="entity-mention-by-entity">
    <xsl:param name="entity-key" required="yes" />

    <xsl:variable name="entry" select="ancestor::tei:text[@xml:id][1]" />

    <!-- Only record occurrences and other entities in entry once per entry, regardless of how many
      keyed elements there are in it. -->
    <xsl:if test="not(preceding::tei:*[@key = $entity-key][ancestor::tei:text[@xml:id] = $entry])">
      <!-- Occurrences. -->
      <xsl:apply-templates mode="occurrence" select=".">
        <xsl:with-param name="entry" select="$entry" />
      </xsl:apply-templates>
    </xsl:if>
  </xsl:template>

  <!-- Occurrences. -->
  <xsl:template match="tei:*[@key]" mode="occurrence">
    <!-- Create a record of an occurrence of this entity in the
         TEI. The format of this field is:
         
         <document ID>::<text ID>::<text heading>
         
         (Non-literals in <>) -->
    <xsl:param name="entry" required="yes" />

    <field name="occurrence">
      <xsl:value-of select="ancestor::tei:TEI/@xml:id" />
      <xsl:text>::</xsl:text>
      <xsl:value-of select="$entry/@xml:id" />
      <xsl:text>::</xsl:text>
      <xsl:value-of select="$entry/tei:body/tei:head[@type = 'main']" />
    </field>
  </xsl:template>
</xsl:stylesheet>
